/**
 * Websocket protocol handler.
 */
;
(function () {

	window._ = {};
	_.Api = {};
	_.Api.websocket =
	{


		/**
		 * URI of the websocket server.
		 * @type {String}
		 */
		_serverUri: null,
		/**
		 * The socket.
		 * @type {Object}
		 */
		_socket: null,
		/**
		 * State of things.
		 * @type {String}
		 */
		_state: 'closed',
		/**
		 * Latency function to estimate latency and keep connection alive.
		 * @type {Function}
		 */
		_latencyChecker: null,
		/**
		 * Latency check start time.
		 * @type {Date}
		 */
		_pingStart: null,
		/**
		 * Latency.
		 * @type {Array}
		 */
		_latencies: [],
		/**
		 * Latency.
		 * @type {Integer}
		 */
		_latencyAverage: 0,
		/**
		 * Message processor callback.
		 * @type {Function}
		 */
		_messageCallback: function(){},
		/**
		 * Connection opened callback.
		 * @type {Function}
		 */
		_connectionOpenedCallback: function(){},
		/**
		 * Connection closed callback.
		 * @type {Function}
		 */
		_connectionClosedCallback: function(){},


		/**
		 * Initialize instance.
		 * @return {}
		 */
        _initialize: function ()
        {
            //-- Establish server uri and connect.
            bug.info('Websocket', '_initialize', 'Initializing websocket submodule.');

            //_.Api.websocket._serverUri = 'ws://' + window.location.hostname + ':' + window.location.port;
            _.Api.websocket._serverUri = 'ws://' + window.location.hostname;
            bug.dump('Websocket', '_initialize', 'Server URI: ' + _.Api.websocket._serverUri);
            _.Api.websocket._state = 'open';
            _.Api.websocket.connect();
        },

		/**
		 * Connect to websocket server.
		 */
		connect: function ()
		{
			//-- Create a new websocket instance.
			_.Api.websocket._socket = new WebSocket(_.Api.websocket._serverUri);

			//-- Handle open event.
			_.Api.websocket._socket.onopen = function (ev)
			{
				_.Api.websocket._state = 'open';
				_.Api.websocket._latencyChecker = setInterval(function ()
				{
					if (WebSocket.OPEN == _.Api.websocket._socket.readyState)
					{
						_.Api.websocket._pingStart = new Date();
						_.Api.websocket._socket.send('{"ping":true}');
					}
				}, 540000);
				_.Api.websocket._connectionOpenedCallback();
				bug.info('Websocket', '_connect', 'Connection established with ' + _.Api.websocket._serverUri + '.');
			}

			//-- Handle message event.
			_.Api.websocket._socket.onmessage = function (ev)
			{
				var message = JSON.parse(ev.data);
				bug.info('Websocket', 'received', message);
				if (message.pong)
				{
					var _pingEnd = new Date();
					_.Api.websocket._latencies.push((_pingEnd.getTime() - _.Api.websocket._pingStart.getTime()) / 2);
					if (10 < _.Api.websocket._latencies.length)
					{
						_.Api.websocket._latencies.shift();
					}
					var sum = _.Api.websocket._latencies.reduce(function (a, b) { return a + b; });
					_.Api.websocket._latencyAverage = sum / _.Api.websocket._latencies.length;
					bug.info('Websocket', 'latency', _.Api.websocket._latencyAverage);
				}
				else
				{
					_.Api.websocket._messageCallback(message);
				}
			};

			//-- Handle error event.
			_.Api.websocket._socket.onerror = function (ev)
			{
				bug.warn('Websocket', '_connect', ev);
			};

			//-- Handle close event.
			_.Api.websocket._socket.onclose = function (ev)
			{
				_.Api.websocket._socket = null;
				clearInterval(_.Api.websocket._latencyChecker);
				_.Api.websocket._connectionClosedCallback();
				if ('closed' == _.Api.websocket._state)
				{
					return;
				}
				bug.info('Websocket', '_connect.onclose', 'Connection closed.');
				bug.dump('Websocket', '_connect.onclose', ev);
				setTimeout(function () {
						$.proxy(_.Api.websocket.connect, _.Api.websocket)()
					},
					15000
				);
			};
		},

		close: function ()
		{
			_.Api.websocket._state = 'closed';
			_.Api.websocket._socket.close();
		}


	};

})();